"""
Digital HandWrite Recognize
~~~~~~~~~~~~~~~~~~~

An easy module for handwriting machinelearning.

:copyright: (c) 2025-present littlecommandcat
:license: MIT, see LICENSE for more details.

"""

from .recognizer import *
from .errors import *
from .decorators import *

__title__ = 'handwriting_recognizer'
__author__ = 'littlecommandcat'
__license__ = 'MIT'
__copyright__ = 'Copyright 2025-present littlecommandcat'
__version__ = '1.2.5'